@@ -568,9 +568,15 @@ case object MinuteDateType extends DateType {
 sealed trait Performances
 case class MailTo(who: String, text: Option[String]) extends Performances
 case class HttpTo(url: String, user: String, password: String, headers: List[(String, String)], data: Option[String]) extends Performances
-case class FetchFeed(url: UrlStore, tags: List[String]) extends Performances
 case class FetchAtom(override val url: UrlStore, override val tags: List[String]) extends FetchFeed(url, tags)
 case class FetchRss(override val url: UrlStore, override val tags: List[String]) extends FetchFeed(url, tags)
 case object PerformResend extends Performances
 case object PerformFilter extends Performances
 case object ScalaInterpret extends Performances
+
+object FetchFeed {
+  def unapply(f: FetchFeed): Option[(UrlStore, List[String])] =
+    Some((f.url, f.tags))
+}
+
+abstract class FetchFeed(val url: UrlStore, val tags: List[String]) extends Performances
\ No newline at end of file
