@@ -24,7 +24,7 @@ import mapper._
 import util._
 import common._
 
-object ExtSession extends ExtSession with MetaProtoExtendedSession[ExtSession] {
+object ExtSession extends ExtSession with LongKeyedMetaMapper[ExtSession] with MetaProtoExtendedSession[ExtSession] {
   override def dbTableName = "ext_session" // define the DB table name
 
   def logUserIdIn(uid: String): Unit = User.logUserIdIn(uid)
@@ -34,7 +34,7 @@ object ExtSession extends ExtSession with MetaProtoExtendedSession[ExtSession] {
   type UserType = User
 }
 
-class ExtSession extends ProtoExtendedSession[ExtSession] {
+class ExtSession extends LongKeyedMapper[ExtSession] with ProtoExtendedSession[ExtSession] {
   def getSingleton = ExtSession // what's the "meta" server
 
 }
