@@ -22,12 +22,14 @@ package org.apache.esme.model
 import net.liftweb._
 import mapper._
 import util._
+import textile.TextileParser._
 import common._
 import http._
 import js._
 import Helpers._
 
 import scala.xml._
+import scala.xml.transform.{RuleTransformer, RewriteRule}
 
 import java.util.regex.Matcher
 
@@ -41,11 +43,9 @@ import org.tartarus.snowball.ext.PorterStemmer
 import org.apache.esme._
 import lib._
 
-object Message extends Message with LongKeyedMetaMapper[Message] {
+object Message extends Message with LongKeyedMetaMapper[Message] with Logger {
   val logger: Logger = Logger ("org.apache.esme.model.Message")
-
-  val root = LiftRules.context.path
-
+                                      
   private def fixConversation(msg: Message) {
     if (!msg.conversation.defined_? && msg.replyTo.defined_?) {
       for (replyTo <- msg.replyTo.obj) {
@@ -55,7 +55,9 @@ object Message extends Message with LongKeyedMetaMapper[Message] {
         msg.conversation(replyTo.conversation)
       }
     }
-  }
+  }      
+  
+  val root = LiftRules.context.path
 
   def cacheSize: Int = 10000
 
@@ -269,9 +271,8 @@ object Message extends Message with LongKeyedMetaMapper[Message] {
     stemmer.stem()
     stemmer.getCurrent()
   }
-  
+
   def transformBody(ns: NodeSeq) = {
-    import scala.xml.transform.{RuleTransformer, RewriteRule}
     toXml.map(new RuleTransformer(new RewriteRule{
       override def transform(n: Node) = n match {
         case e: Elem if "body" == e.label => <body>{ns}</body>
@@ -279,6 +280,7 @@ object Message extends Message with LongKeyedMetaMapper[Message] {
       }
     })).head
   }
+
 }
 
 
@@ -383,8 +385,9 @@ class Message extends LongKeyedMapper[Message] with ManyToMany {
 
   }
 
-  lazy val digestedXHTML = {
-    (toXml \ "body").flatMap(_.child map {
+  lazy val msgInfoTransformer = new RuleTransformer(
+    new RewriteRule {
+      override def transform(n: Node) = n match {
         case e: Elem if e.label == "at_name" =>
           e.attribute("nickname").
           map(nickname =>
@@ -398,35 +401,26 @@ class Message extends LongKeyedMapper[Message] with ManyToMany {
 
         case e: Elem if e.label == "url" =>
           e.attribute("url").flatMap(url =>
-            e.attribute("uniqueId").map { id =>
-              val href =
-                if (pool.defined_?)
-                  // disable shortener to avoid popularity statistics
-                  url.toString
-                else
-                  "/u/" + id
-              def truncateUrl(urlString: String, len: Int) =
-                if (urlString.length <= len)
-                  urlString
-                else
-                  urlString.substring(0, len - 3) + "..."
-              <xml:group> <a class="tiplelement" href={root+href} target="_blank" title={url}>{truncateUrl(url.toString, 20)}</a> </xml:group>
-            }
+            e.attribute("name").flatMap(name =>
+              e.attribute("uniqueId").map { id =>
+                val href =
+                  if (pool.defined_?)
+                    // disable shortener to avoid popularity statistics
+                    url.toString
+                  else
+                    root + "/u/" + id
+                <xml:group> <a class="tiplelement" href={href} target="_blank" title={name}>{name}</a> </xml:group>
+              }
+            )
           ).getOrElse(Text("") )
 
-        case e: Elem if e.label == "em" =>
-          e.attribute("text").map(text =>
-            <em>{text}</em>).
-          getOrElse(e)
-        
-        case e: Elem if e.label == "strong" =>
-          e.attribute("text").map(text =>
-            <strong>{text}</strong>).
-          getOrElse(e)
-        
         case x => x
-      })
-  }
+      }
+    }
+  )
+
+  lazy val digestedXHTML =
+    (toXml \ "body").map(msgInfoTransformer).flatMap(_.child)
 
   private lazy val originalXml = PCDataXmlParser(text.is).openOr(<message/>)
   
@@ -523,18 +517,12 @@ class Message extends LongKeyedMapper[Message] with ManyToMany {
       lst =>
       val xml = <message><body>{
             lst map {
-              case HashTag(t) => t.toXml
-              case AtName(user) => <at_name id={user.id.toString}
-                  nickname={user.nickname.is} >{"@" + user.nickname.is}</at_name>
-              case MsgText(text) => Text(text)
-              case URL(url) => <url id={url.id.toString}
-                  url={url.url.toString} uniqueId={url.uniqueId.is} >{url.url.toString}</url>
-              case Emph(text) => <xml:group> <em text={text}>_{text}_</em></xml:group>
-              case Strong(text) => <xml:group> <strong text={text}>*{text}*</strong></xml:group>
+              // get rid of extra paragraphs
+              case textile: Textile => paraFixer(textile.toHtml)
             }
           }</body>
         <tags>{
-            ((lst.flatMap{case HashTag(t) => Full(t) case _ => Empty})
+            ((lst.flatMap{case Paragraph(elems, _) => {elems.flatMap { case HashTag(t) => Full(t) case _ => Empty} }  case _ => Empty})
              ::: tags).distinct.map(_.toXml)
           }</tags>{
           metaData match {
@@ -543,6 +531,7 @@ class Message extends LongKeyedMapper[Message] with ManyToMany {
           }
 
         }</message>
+      debug("Message.setTextAndTags(), generated XML: %s".format(xml.toString))
       this.text(xml.toString)
       this
     }
